﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NativeChart = Syncfusion.UI.Xaml.Charts;
using System.Threading.Tasks;
using Xamarin.Forms.Platform.UWP;
using Syncfusion.SfChart.XForms.UWP;
using Windows.UI.Xaml.Markup;
using Windows.UI.Xaml;
using Syncfusion.SfChart.XForms;
using System.Collections.ObjectModel;
using Windows.Foundation;
using Windows.UI.Xaml.Input;
using Windows.Devices.Input;
using Windows.UI.Xaml.Shapes;
using SyncfusionXamarinApp1;


[assembly: ExportRenderer(typeof(CustomChart), typeof(SyncfusionXamarinApp1.UWP.CustomRenderer))]
namespace SyncfusionXamarinApp1.UWP
{
    public class CustomRenderer : SfChartRenderer
    {
        int index;
        double X, Y, stackedValues;
        bool isPressed;

        protected override void OnElementChanged(ElementChangedEventArgs<SfChart> e)
        {
            base.OnElementChanged(e);

            Control.PointerPressed += Control_PointerPressed;
            Control.PointerMoved += Control_PointerMoved;
            Control.PointerReleased += Control_PointerReleased;

            if (e.OldElement != null)
            {
                Control.PointerPressed -= Control_PointerPressed;
                Control.PointerMoved -= Control_PointerMoved;
                Control.PointerReleased -= Control_PointerReleased;
            }


            CustomTrackBall TrackBallBehavior = new CustomTrackBall();
            TrackBallBehavior.ShowLine = false;
            TrackBallBehavior.UseSeriesPalette = true;
            Control.Series[0].TrackBallLabelTemplate = SyncfusionXamarinApp1.UWP.App.Current.Resources["LabelTemplate"] as DataTemplate;
            Control.Behaviors.Add(TrackBallBehavior);
        }

        private void Control_PointerReleased(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            isPressed = false;
        }

        private void Control_PointerMoved(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            if (Control.Series.Count <= 0 || index < 0  ||!isPressed)
                return;
            Control.Series[0].ListenPropertyChange = true;
            double xValue, yValue;
            var point = e.GetCurrentPoint(Control);
            double x = point.Position.X - Control.SeriesClipRect.Left;
            double y = point.Position.Y - Control.SeriesClipRect.Top;
            Point chartPoint = new Point(x, y);

            var datas = Control.Series[0].ItemsSource as ObservableCollection<Model>;
            if (datas.Count > index && !(index < 0))
            {
                xValue = Control.PointToValue(Control.PrimaryAxis, chartPoint);
                yValue = Control.PointToValue(Control.SecondaryAxis, chartPoint);
                datas[index].Date = xValue;
                datas[index].Value = yValue;
            }
        }

        private void Control_PointerPressed(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            //if ((e.OriginalSource is Line && (e.OriginalSource as Line).Tag !=null) || (e.OriginalSource is Ellipse))
            if (e.OriginalSource is Ellipse)
            {
                isPressed = true;
                var point = e.GetCurrentPoint(Control);
                double x = point.Position.X - Control.SeriesClipRect.Left;
                double y = point.Position.Y - Control.SeriesClipRect.Top;
                Point chartPoint = new Point(x, y);

                if (Control.Series.Count > 0)
                {
                    Control.Series[0].FindNearestChartPoint(chartPoint, out X, out Y, out stackedValues);
                    var model = Control.Series[0].ItemsSource as ObservableCollection<Model>;
                    for (int i = 0; i < model.Count; i++)
                    {
                        if (!double.IsNaN(X) && model[i].Date == X)
                        {
                            index = i;
                            return;
                        }

                    }
                }
            }
        }
    }

    public class CustomTrackBall : NativeChart.ChartTrackBallBehavior
    {
        bool isPressed;
        protected override void OnPointerPressed(PointerRoutedEventArgs e)
        {
            //if ((e.OriginalSource is Line && (e.OriginalSource as Line).Tag != null) || (e.OriginalSource is Ellipse))
            if (e.OriginalSource is Ellipse)
            {
                IsActivated = true;
                isPressed = true;
                CustomPointerPositionChanged();
            }
        }

        protected override void OnPointerPositionChanged()
        {
            if (!isPressed && IsActivated)
                IsActivated = false;
            else
                base.OnPointerPositionChanged();
        }

        private void CustomPointerPositionChanged()
        {
            OnPointerPositionChanged();
        }

        protected override void OnPointerReleased(PointerRoutedEventArgs e)
        {

            if (e.Pointer.PointerDeviceType != PointerDeviceType.Mouse)
                base.OnPointerReleased(e);
            else if (e.Pointer.PointerDeviceType == PointerDeviceType.Mouse && IsActivated)
            {
                IsActivated = false;

            }
            isPressed = false;
            this.ClearItems();
        }

    }
}
