﻿using Syncfusion.SfChart.XForms;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace SyncfusionXamarinApp1
{
    public class SampleChart : ContentPage
    {
        ViewModel dataModel;

        CustomChart chart;

        public SampleChart()
        {
            StackLayout layout = new StackLayout() { Padding = new Thickness(5, 15, 5, 5) };

            dataModel = new ViewModel();

            chart = new CustomChart();
            chart.VerticalOptions = LayoutOptions.FillAndExpand;
            chart.HorizontalOptions = LayoutOptions.FillAndExpand;
            chart.BindingContext = dataModel;

            /* !!!! change from NumericalAxis to LogarithmicAxis here !!! */
            LogarithmicAxis datetimeAxis = new LogarithmicAxis();
            datetimeAxis.Minimum = 20;
            datetimeAxis.Maximum = 20000;
            chart.PrimaryAxis = datetimeAxis;

            NumericalAxis numericalAxis = new NumericalAxis();
            numericalAxis.Minimum = -20;
            numericalAxis.Maximum = 20;
            chart.SecondaryAxis = numericalAxis;

            /* !!!! change from LineSeries to ScatterSeries here !!! 
               When use ScatterSeries on Android, ChartSelectionBehaviorExt doesn't work anymore with output message
                W/monodroid-assembly( 6426): typemap: unable to find mapping to a managed type from Java type 'android/view/DisplayListCanvas'
                W/monodroid-assembly( 6426): typemap: unable to find mapping to a managed type from Java type 'android/view/RecordingCanvas' */
            //chart.Series.Add(new ScatterSeries()
            chart.Series.Add(new LineSeries()
            {
                DataMarker = new ChartDataMarker
                {
                    ShowMarker = true,
                    ShowLabel = false,
                    MarkerWidth = 20,
                    MarkerHeight = 20,
                    MarkerColor = Color.FromHex("#FFFEBE17"),
                
                },
                ListenPropertyChange = true,
                ItemsSource = dataModel.DateTimeData,
                XBindingPath = "Date",
                YBindingPath = "Value",
            });

            ChartSelectionBehaviorExt selectionBehavior = new ChartSelectionBehaviorExt(chart, dataModel.DateTimeData);
            chart.ChartBehaviors.Add(selectionBehavior);
            
            if(Device.RuntimePlatform != Device.UWP)
            {
                ChartTrackballBehaviorExt trackball = new ChartTrackballBehaviorExt();
                trackball.ShowLine = false;
                chart.ChartBehaviors.Add(trackball);
            }

            layout.Children.Add(chart);
            Content = layout;
        }
    }

    public class ChartTrackballBehaviorExt : ChartTrackballBehavior
    {
        protected override void OnTouchMove(float x, float y)
        {
            base.OnTouchMove(x, y);
            Show(x, y);
        }

        protected override void OnTouchUp(float x, float y)
        {
            base.OnTouchUp(x, y);
            Hide();
        }
    }

    public class ChartSelectionBehaviorExt : ChartSelectionBehavior
    {
        CustomChart chart;
        ObservableCollection<Model> datas;
        int index = -1;

        public ChartSelectionBehaviorExt(CustomChart chart, ObservableCollection<Model> data)
        {
            this.chart = chart;
            this.datas = data;
        }

        protected override void OnTouchDown(float x, float y)
        {
            base.OnTouchDown(x, y);

            if (chart.Series.Count > 0)
            {
                index = chart.Series[0].GetDataPointIndex(x, y);
            }
        }

        protected override void OnTouchMove(float x, float y)
        {
            base.OnTouchMove(x, y);

            if (chart.Series.Count <= 0 || index < 0)
                return;

            double XValue, YValue;
            Point chartPoint = new Point(x, y);

            if (datas.Count > index && !(index < 0))
            {
                /* !!!! On Android LineSeries, chart.PointToValue with LogarithmicAxis return inf !!! 
                        On Android ScatterSeries, it cannot even reach this line 
                        On UWP LineSeries or ScatterSeries, it works with slow performance 
                        On Android LineSeries, with NumericalAxis on both Primary and Secondary, it works with better performance than UWP */
                XValue = chart.PointToValue(chart.PrimaryAxis, chartPoint);
                YValue = chart.PointToValue(chart.SecondaryAxis, chartPoint);

                datas[index].Date = XValue;
                datas[index].Value = YValue;
            }
        }

        protected override void OnTouchUp(float x, float y)
        {
            base.OnTouchUp(x, y);

            index = -1;
        }
    }
}

